let isOpen = false;

window.onload = mainDo;

async function open(callback) {
    let serverUrl = document.getElementById("serverUrl").value;
    if (!serverUrl) {
        alert('serverUrl cannot be empty!');
        return;
    }
    await SocketD.createClient(serverUrl.trim())
        .config(c => c
            .heartbeatInterval(1000*5)
            .fragmentSize(1024 * 1024)
            .metaPut("test","1"))
        .connectHandler(c=> {
            console.log("connect begin...");
            c.getConfig().metaPut("test","1");
            return c.connect();
        })
        .listen(SocketD.newEventListener()
            .doOnOpen(s=>{
                window.clientSession = s;
                console.log("outmeta: test=" + s.handshake().param("test"))
            })
            .doOnMessage((s, m) => {
            appendToMessageList('Receive push', m.dataAsString());
        }))
        .open();
    console.log('session', clientSession);
    if (callback) callback();
}

function close(callback) {
    clientSession.close();
    if (callback) callback();
}

function send(type) {
    let input = document.getElementById("input").value;
    if (!input) {
        alert("Input message cannot be empty!");
        return;
    }


    if (type == 1) {
        appendToMessageList('Send and request', input);
        clientSession.sendAndRequest("/login", SocketD.newEntity(input)).thenReply(reply => {
            console.log('reply', reply);
            appendToMessageList('reply', reply.dataAsString());
        });
    } else if (type == 2) {
        appendToMessageList('Send and subscribe', input);
        clientSession.sendAndSubscribe("/demo", SocketD.newEntity(input)
            .metaPut(SocketD.EntityMetas.META_RANGE_SIZE,"3")).thenReply(reply => {
            console.log('reply', reply);
            if(reply.isEnd()){
                appendToMessageList('End of reply', reply.dataAsString());
            }else{
                appendToMessageList('reply', reply.dataAsString());
            }
        });
    } else {
        appendToMessageList('Login', input);
        clientSession.send("/room1", SocketD.newEntity(input));
    }
}

function appendToMessageList(hint, msg) {
    let ele = document.getElementById("message");
    ele.value = `[${dateFormat(new Date(), 'yyyy-MM-dd hh:mm:ss.SSS')}] ${hint}：${msg}\n` + ele.value;

}



function mainDo() {
    let openBtn = document.getElementById("openBtn");
    openBtn.addEventListener("click", function () {
        if (isOpen) {
            close(() => {
                openBtn.innerHTML = 'connect';
                isOpen = false;
            });
        } else {
            open(() => {
                openBtn.innerHTML = 'closure';
                isOpen = true;
            });
        }

    });
    let send0 = document.getElementById("send");
    let send1 = document.getElementById("sendAndRequest");
    let send2 = document.getElementById("sendAndSubscribe");
    send0.addEventListener("click", () => {
        if (isOpen) {
            send(0);
        }
    });

    send1.addEventListener("click", () => {
        if (isOpen) {
            send(1);
        }
    });

    send2.addEventListener("click", () => {
        if (isOpen) {
            send(2);
        }
    });


    let push = document.getElementById("push");
    let unpush = document.getElementById("unpush");

    push.addEventListener("click", () => {
        if (isOpen) {
            clientSession.send("/room1", SocketD.newEntity());
        }
    });

    unpush.addEventListener("click", () => {
        if (isOpen) {
            clientSession.send("/unpush", SocketD.newEntity())
        }
    });


    let uploadFile = document.getElementById("uploadFile");
    let downloadFile = document.getElementById("downloadFile");
    let uploadData = document.getElementById("uploadData");
    uploadFile.addEventListener("click", async () => {
        if (isOpen) {
            const files = document.getElementById("file").files;
            if (!files || files.length == 0) {
                alert("Please select file");
                return;
            }

            const file1 = document.getElementById("file").files[0];

            appendToMessageList('Send files and request', file1.name);					
            clientSession.sendAndRequest("/sendfile", SocketD.newEntity(file1).metaPut("FILE_NAME",file1.name), 100_000).thenReply(reply => {
                console.log('reply', reply);
                appendToMessageList('reply', reply.dataAsString());
            }).thenProgress((isSend, val, max) => {
                if (isSend) {
                    appendToMessageList('Send progress', val + "/" + max);
                }
            });
        }
    });

    downloadFile.addEventListener("click", async () => {
        if (isOpen) {
            appendToMessageList('download file', "...");
            clientSession.sendAndRequest("/download", SocketD.newEntity()).thenReply(reply => {
                console.log('reply', reply);

                const fileName = reply.meta(SocketD.EntityMetas.META_DATA_DISPOSITION_FILENAME);
                if (fileName) {
                    appendToMessageList('download file', "file=" + fileName + ", size=" + reply.dataSize());
                } else {
                    appendToMessageList('download file', "No files received :(");
                }
            }).thenProgress((isSend,val,max)=>{
                if(!isSend){
                    appendToMessageList('Download progress', val + "/" + max);
                }
            });
        }
    });

    uploadData.addEventListener("click", async () => {
        if (isOpen) {
            const strSize = 1024*1024*10;
            let str = "";
            while (str.length < strSize) {
                str += "1234567890"
            }

            appendToMessageList('Submit large text block 10M', "...");
            clientSession.sendAndRequest("/upload", SocketD.newEntity(str)).thenReply(reply => {
                console.log('reply', reply);
                appendToMessageList('reply', reply.dataAsString());
            }).thenProgress((isSend, val,max)=> {
                if(isSend){
                    appendToMessageList('Submit progress', val + "/" + max);
                }
            });
        }
    });
}

//Date formatting
function dateFormat(date,fmt) {
    //default format
    fmt = fmt ? fmt : 'yyyy-MM-dd hh:mm:ss';

    var o = {
        "M+" : date.getMonth()+1,                 
        "d+" : date.getDate(),                    
        "h+" : date.getHours(),                   
        "m+" : date.getMinutes(),                 
        "s+" : date.getSeconds(),                
        "q+" : Math.floor((date.getMonth()+3)/3), 
        "S+"  : date.getMilliseconds()             
    };
    if(/(y+)/.test(fmt)) {
        fmt=fmt.replace(RegExp.$1, (date.getFullYear()+"").substr(4 - RegExp.$1.length));
    }
    for(var k in o) {
        if(new RegExp("("+ k +")").test(fmt)){
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) :
                RegExp.$1.length==2 ? (("00"+ o[k]).substr((""+ o[k]).length)) : (("000"+ o[k]).substr((""+ o[k]).length))
            );
        }
    }
    return fmt;
}
